@@ -9,6 +9,7 @@
 
 #include "common/Error.h"
 
+#include <QtCore/QPointer>
 #include <QtWidgets/QMessageBox>
 
 AchievementLoginDialog::AchievementLoginDialog(QWidget* parent, Achievements::LoginRequestReason reason)
@@ -16,7 +17,8 @@ AchievementLoginDialog::AchievementLoginDialog(QWidget* parent, Achievements::Lo
 	, m_reason(reason)
 {
 	m_ui.setupUi(this);
-	QtUtils::SetScalableIcon(m_ui.loginIcon, QIcon::fromTheme(QStringLiteral("login-box-line")), QSize(32, 32));
+	const QString base_path(QtHost::GetResourcesBasePath());
+	QtUtils::SetScalableIcon(m_ui.loginIcon, QIcon(QStringLiteral("%1/icons/ra-icon.svg").arg(base_path)), QSize(50, 50));
 	setWindowFlags(windowFlags() & ~Qt::WindowContextHelpButtonHint);
 
 	// Adjust text if needed based on reason.
@@ -44,11 +46,18 @@ void AchievementLoginDialog::loginClicked()
 	m_ui.status->setText(tr("Logging in..."));
 	enableUI(false);
 
-	Host::RunOnCPUThread([this, username = std::move(username), password = std::move(password)]() {
+	const QPointer<AchievementLoginDialog> dialog(this);
+	Host::RunOnCPUThread([dialog, username = std::move(username), password = std::move(password)]() {
 		Error error;
 		const bool result = Achievements::Login(username.c_str(), password.c_str(), &error);
 		const QString message = QString::fromStdString(error.GetDescription());
-		QMetaObject::invokeMethod(this, "processLoginResult", Qt::QueuedConnection, Q_ARG(bool, result), Q_ARG(const QString&, message));
+
+		QtHost::RunOnUIThread([dialog, result, message = std::move(message)]() {
+			if (!dialog)
+				return;
+
+			dialog->processLoginResult(result, message);
+		});
 	});
 }
 
@@ -119,7 +128,25 @@ void AchievementLoginDialog::processLoginResult(bool result, const QString& mess
 		}
 	}
 
-	done(0);
+	// Show success messagebox
+	const std::string username = Host::GetBaseStringSettingValue("Achievements", "Username");
+	QMessageBox::information(
+		this, tr("Login Successful"),
+		tr("Successfully logged in to RetroAchievements as %1.").arg(QString::fromStdString(username)));
+	
+	m_ui.status->setText(tr("Successfully logged in as %1.").arg(QString::fromStdString(username)));
+	m_ui.status->setStyleSheet("color: green; font-weight: bold;");
+	
+	disconnect(m_ui.buttonBox, &QDialogButtonBox::accepted, this, &AchievementLoginDialog::loginClicked);
+	
+	m_login->setVisible(false);
+	QPushButton* dismissButton = m_ui.buttonBox->addButton(tr("&Dismiss"), QDialogButtonBox::AcceptRole);
+	dismissButton->setDefault(true);
+	dismissButton->setFocus();
+	
+	connect(dismissButton, &QPushButton::clicked, this, [this]() { done(0); });
+	
+	enableUI(false);
 }
 
 void AchievementLoginDialog::connectUi()
