@@ -985,10 +985,6 @@ public:
 	__forceinline GSVector4i modulate16(const GSVector4i& f) const
 	{
 		// a * f << shift
-		if (shift == 0)
-		{
-			return mul16hrs(f);
-		}
 
 		return sll16<shift + 1>().mul16hs(f);
 	}
