@@ -23,8 +23,8 @@ public:
 		bool m_complete;
 		bool m_repeating;
 		std::vector<GSVector2i>* m_p2t;
-		u32 m_valid[MAX_PAGES];
-		std::array<u16, MAX_PAGES> m_erase_it;
+		u32 m_valid[GS_MAX_PAGES];
+		std::array<u16, GS_MAX_PAGES> m_erase_it;
 		const u32* RESTRICT m_sharedbits;
 
 		// m_valid
@@ -42,7 +42,7 @@ public:
 
 protected:
 	std::unordered_set<Texture*> m_textures;
-	std::array<FastList<Texture*>, MAX_PAGES> m_map;
+	std::array<FastList<Texture*>, GS_MAX_PAGES> m_map;
 
 public:
 	GSTextureCacheSW();
