@@ -63,10 +63,6 @@ namespace Host
 	void ReportErrorAsync(const std::string_view title, const std::string_view message);
 	void ReportFormattedErrorAsync(const std::string_view title, const char* format, ...);
 
-	/// Displays a synchronous confirmation on the UI thread, i.e. blocks the caller.
-	bool ConfirmMessage(const std::string_view title, const std::string_view message);
-	bool ConfirmFormattedMessage(const std::string_view title, const char* format, ...);
-
 	/// Sets batch mode (exit after game shutdown).
 	bool InBatchMode();
 
@@ -138,6 +134,8 @@ namespace Host
 
 	/// Direct access to settings interface. Must hold the lock when calling GetSettingsInterface() and while using it.
 	std::unique_lock<std::mutex> GetSettingsLock();
+	/// Ditto for secrets file.
+	std::unique_lock<std::mutex> GetSecretsSettingsLock();
 	SettingsInterface* GetSettingsInterface();
 
 	/// Sets host-specific default settings.
@@ -151,6 +149,9 @@ namespace Host
 		/// Retrieves the base settings layer. Must call with lock held.
 		SettingsInterface* GetBaseSettingsLayer();
 
+		/// Retrieves the base settings layer. Must call with lock held.
+		SettingsInterface* GetSecretsSettingsLayer();
+
 		/// Retrieves the game settings layer, if present. Must call with lock held.
 		SettingsInterface* GetGameSettingsLayer();
 
@@ -160,6 +161,9 @@ namespace Host
 		/// Sets the base settings layer. Should be called by the host at initialization time.
 		void SetBaseSettingsLayer(SettingsInterface* sif);
 
+		/// Sets the secrets settings layer. Should follow call to SetBaseSettingsLayer.
+		void SetSecretsSettingsLayer(SettingsInterface* sif);
+
 		/// Sets the game settings layer. Called by VMManager when the game changes.
 		void SetGameSettingsLayer(SettingsInterface* sif, std::unique_lock<std::mutex>& settings_lock);
 
