@@ -4,12 +4,19 @@
 #pragma once
 
 #include "ImGuiManager.h"
+#include "Config.h"
+
+struct ImVec2;
 
 namespace ImGuiManager
 {
 	void RenderOverlays();
 }
 
+ImVec2 CalculateOSDPosition(OsdOverlayPos position, float margin, const ImVec2& text_size, float window_width, float window_height);
+ImVec2 CalculatePerformanceOverlayTextPosition(OsdOverlayPos position, float margin, const ImVec2& text_size, float window_width, float position_y);
+bool ShouldUseLeftAlignment(OsdOverlayPos position);
+
 namespace SaveStateSelectorUI
 {
 	static constexpr float DEFAULT_OPEN_TIME = 5.0f;
