@@ -1,22 +1,10 @@
-/*  PCSX2 - PS2 Emulator for PCs
- *  Copyright (C) 2002-2010  PCSX2 Dev Team
- *
- *  PCSX2 is free software: you can redistribute it and/or modify it under the terms
- *  of the GNU Lesser General Public License as published by the Free Software Found-
- *  ation, either version 3 of the License, or (at your option) any later version.
- *
- *  PCSX2 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
- *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
- *  PURPOSE.  See the GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License along with PCSX2.
- *  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2002-2025 PCSX2 Dev Team
+// SPDX-License-Identifier: GPL-3.0+
 
-#include "PrecompiledHeader.h"
 #include "IopHw_Internal.h"
 #include "Sif.h"
-#include "Sio.h"
+#include "SIO/Sio2.h"
+#include "SIO/Sio0.h"
 #include "FW.h"
 #include "CDVD/Ps1CD.h"
 #include "SPU2/spu2.h"
@@ -86,7 +74,7 @@ void iopHwWrite8_Page1( u32 addr, mem8_t val )
 	switch( masked_addr )
 	{
 		case (HW_SIO_DATA & 0x0fff):
-			sio0.SetTxData(val);
+			g_Sio0.SetTxData(val);
 			break;
 		case (HW_SIO_STAT & 0x0fff):
 			Sio0Log.Error("%s(%08X, %08X) Unexpected SIO0 STAT 8 bit write", __FUNCTION__, addr, val);
@@ -116,12 +104,12 @@ void iopHwWrite8_Page1( u32 addr, mem8_t val )
 				DbgCon.Warning( "HwWrite8 to Counter16 [ignored] @ addr 0x%08x = 0x%02x", addr, psxHu8(addr) );
 				psxHu8( addr ) = val;
 			}
-			else if( masked_addr >= 0x480 && masked_addr < 0x4a0 )
+			else if ( masked_addr >= 0x480 && masked_addr < 0x4a0 )
 			{
 				DbgCon.Warning( "HwWrite8 to Counter32 [ignored] @ addr 0x%08x = 0x%02x", addr, psxHu8(addr) );
 				psxHu8( addr ) = val;
 			}
-			else if( (masked_addr >= pgmsk(HW_USB_START)) && (masked_addr < pgmsk(HW_USB_END)) )
+			else if ( (masked_addr >= pgmsk(HW_USB_START)) && (masked_addr < pgmsk(HW_USB_END)) )
 			{
 				USBwrite8( addr, val );
 			}
@@ -140,7 +128,7 @@ void iopHwWrite8_Page3( u32 addr, mem8_t val )
 	// all addresses are assumed to be prefixed with 0x1f803xxx:
 	pxAssert( (addr >> 12) == 0x1f803 );
 
-	if( SysConsole.iopConsole.IsActive() && (addr == 0x1f80380c) )	// STDOUT
+	if(ConsoleLogging.iopConsole.IsActive() && (addr == 0x1f80380c))	// STDOUT
 	{
 		static char pbuf[1024];
 		static int pidx;
@@ -176,7 +164,7 @@ void iopHwWrite8_Page8( u32 addr, mem8_t val )
 
 	if (addr == HW_SIO2_DATAIN)
 	{
-		sio2.Write(val);
+		g_Sio2.Write(val);
 	}
 	else
 	{
@@ -231,7 +219,7 @@ static __fi void _HwWrite_16or32_Page1( u32 addr, T val )
 	// ------------------------------------------------------------------------
 	// Counters, 32-bit varieties!
 	//
-	else if( masked_addr >= 0x480 && masked_addr < 0x4b0 )
+	else if ( masked_addr >= 0x480 && masked_addr < 0x4b0 )
 	{
 		int cntidx = (( masked_addr >> 4 ) & 0xf) - 5;
 		switch( masked_addr & 0xf )
@@ -264,14 +252,14 @@ static __fi void _HwWrite_16or32_Page1( u32 addr, T val )
 	// ------------------------------------------------------------------------
 	// USB, with both 16 and 32 bit interfaces
 	//
-	else if( (masked_addr >= pgmsk(HW_USB_START)) && (masked_addr < pgmsk(HW_USB_END)) )
+	else if ( (masked_addr >= pgmsk(HW_USB_START)) && (masked_addr < pgmsk(HW_USB_END)) )
 	{
 		if( sizeof(T) == 2 ) USBwrite16( addr, val ); else USBwrite32( addr, val );
 	}
 	// ------------------------------------------------------------------------
 	// SPU2, accessible in 16 bit mode only!
 	//
-	else if( (masked_addr >= pgmsk(HW_SPU2_START)) && (masked_addr < pgmsk(HW_SPU2_END)) )
+	else if ( (masked_addr >= pgmsk(HW_SPU2_START)) && (masked_addr < pgmsk(HW_SPU2_END)) )
 	{
 		if( sizeof(T) == 2 )
 			SPU2write( addr, val );
@@ -284,7 +272,7 @@ static __fi void _HwWrite_16or32_Page1( u32 addr, T val )
 	// ------------------------------------------------------------------------
 	// PS1 GPU access
 	//
-	else if( (masked_addr >= pgmsk(HW_PS1_GPU_START)) && (masked_addr < pgmsk(HW_PS1_GPU_END)) )
+	else if ( (masked_addr >= pgmsk(HW_PS1_GPU_START)) && (masked_addr < pgmsk(HW_PS1_GPU_END)) )
 	{
 		// todo: psx mode: this is new
 		if( sizeof(T) == 2 )
@@ -298,25 +286,22 @@ static __fi void _HwWrite_16or32_Page1( u32 addr, T val )
 	{
 		switch( masked_addr )
 		{
+			case 0x450:
+				psxHu(addr) = val;
+				if (val & (1 << 1))
+				{
+					hwIntcIrq(INTC_SBUS);
+				}
+				break;
 			// ------------------------------------------------------------------------
 			case (HW_SIO_DATA & 0x0fff):
 				Console.Error("%s(%08X, %08X) Unexpected 16 or 32 bit write to SIO0 DATA!", __FUNCTION__, addr, val);
-/*
-				sio0.SetTxData(val & 0xFF);
-				sio0.SetTxData((val >> 8) & 0xFF);
-				
-				if (sizeof(T) == 4)
-				{
-					sio0.SetTxData((static_cast<u32>(val) >> 16) & 0xFF);
-					sio0.SetTxData((static_cast<u32>(val) >> 24) & 0xFF);
-				}
-*/				
 				break;
 			case (HW_SIO_STAT & 0x0fff):
 				Console.Error("%s(%08X, %08X) Write issued to read-only SIO0 STAT!", __FUNCTION__, addr, val);
 				break;
 			case (HW_SIO_MODE & 0x0fff):
-				sio0.SetMode(static_cast<u16>(val));
+				g_Sio0.SetMode(static_cast<u16>(val));
 
 				if (sizeof(T) == 4)
 				{
@@ -326,11 +311,11 @@ static __fi void _HwWrite_16or32_Page1( u32 addr, T val )
 				break;
 
 			case (HW_SIO_CTRL & 0x0fff):
-				sio0.SetCtrl(static_cast<u16>(val));
+				g_Sio0.SetCtrl(static_cast<u16>(val));
 				break;
 			
 			case (HW_SIO_BAUD & 0x0fff):
-				sio0.SetBaud(static_cast<u16>(val));
+				g_Sio0.SetBaud(static_cast<u16>(val));
 				break;
 
 			// ------------------------------------------------------------------------
@@ -612,9 +597,9 @@ void iopHwWrite32_Page8( u32 addr, mem32_t val )
 		{
 			Sio2Log.WriteLn("%s(%08X, %08X) SIO2 SEND3 Write (len = %d / %d) (port = %d)", __FUNCTION__, addr, val, (val >> 8) & Send3::COMMAND_LENGTH_MASK, (val >> 18) & Send3::COMMAND_LENGTH_MASK, val & 0x01);
 			const int parm = (masked_addr - 0x200) / 4;
-			sio2.SetSend3(parm, val);
+			g_Sio2.SetSend3(parm, val);
 		}
-		else if( masked_addr < 0x260 )
+		else if ( masked_addr < 0x260 )
 		{
 			// SIO2 Send commands alternate registers.  First reg maps to Send1, second
 			// to Send2, third to Send1, etc.  And the following clever code does this:
@@ -624,15 +609,15 @@ void iopHwWrite32_Page8( u32 addr, mem32_t val )
 			if (masked_addr & 4)
 			{
 				Sio2Log.WriteLn("%s(%08X, %08X) SIO2 SEND2 Write", __FUNCTION__, addr, val);
-				sio2.send2.at(parm) = val;
+				g_Sio2.send2[parm] = val;
 			}
 			else
 			{
 				Sio2Log.WriteLn("%s(%08X, %08X) SIO2 SEND1 Write", __FUNCTION__, addr, val);
-				sio2.send1.at(parm) = val;
+				g_Sio2.send1[parm] = val;
 			}
 		}
-		else if( masked_addr <= 0x280 )
+		else if ( masked_addr <= 0x280 )
 		{
 			switch( masked_addr )
 			{
@@ -644,31 +629,31 @@ void iopHwWrite32_Page8( u32 addr, mem32_t val )
 					break;
 				case (HW_SIO2_CTRL & 0x0fff):
 					Sio2Log.WriteLn("%s(%08X, %08X) SIO2 CTRL Write", __FUNCTION__, addr, val);
-					sio2.SetCtrl(val);
+					g_Sio2.SetCtrl(val);
 					break;
 				case (HW_SIO2_RECV1 & 0x0fff):
 					Sio2Log.WriteLn("%s(%08X, %08X) SIO2 RECV1 Write", __FUNCTION__, addr, val);
-					sio2.recv1 = val;
+					g_Sio2.recv1 = val;
 					break;
 				case (HW_SIO2_RECV2 & 0x0fff):
 					Sio2Log.WriteLn("%s(%08X, %08X) SIO2 RECV2 Write", __FUNCTION__, addr, val);
-					sio2.recv2 = val;
+					g_Sio2.recv2 = val;
 					break;
 				case (HW_SIO2_RECV3 & 0x0fff):
 					Sio2Log.WriteLn("%s(%08X, %08X) SIO2 RECV3 Write", __FUNCTION__, addr, val);
-					sio2.recv3 = val;
+					g_Sio2.recv3 = val;
 					break;
 				case (HW_SIO2_8278 & 0x0fff):
 					Sio2Log.WriteLn("%s(%08X, %08X) SIO2 UNK1 Write", __FUNCTION__, addr, val);
-					sio2.unknown1 = val;
+					g_Sio2.unknown1 = val;
 					break;
 				case (HW_SIO2_827C & 0x0fff):
 					Sio2Log.WriteLn("%s(%08X, %08X) SIO2 UNK2 Write", __FUNCTION__, addr, val);
-					sio2.unknown2 = val;
+					g_Sio2.unknown2 = val;
 					break;
 				case (HW_SIO2_INTR & 0x0fff):
 					Sio2Log.WriteLn("%s(%08X, %08X) SIO2 ISTAT Write", __FUNCTION__, addr, val);
-					sio2.iStat = val;
+					g_Sio2.iStat = val;
 					break;
 				// Other SIO2 registers are read-only, no-ops on write.
 				default:
@@ -676,7 +661,7 @@ void iopHwWrite32_Page8( u32 addr, mem32_t val )
 				break;
 			}
 		}
-		else if( masked_addr >= pgmsk(HW_FW_START) && masked_addr <= pgmsk(HW_FW_END) )
+		else if ( masked_addr >= pgmsk(HW_FW_START) && masked_addr <= pgmsk(HW_FW_END) )
 		{
 			FWwrite32( addr, val );
 		}
